#include "actor.hpp"
#include <string>
#include <iostream>
#include <fstream>

using namespace std;

Actor::Actor(){
	body_sp = new Sprite();
	arms_sp = new Sprite();
	legs_sp = new Sprite();
	head_sp = new Sprite();
	hair_sp = new Sprite();
	mouth_sp = new Sprite();
	eyes_sp = new Sprite();
	eyeballs_sp = new Sprite();

	pmatrix.scale(0.3322, 0.92, 1.0);
	headmatrix.translate(0.0, 0.0, 0.0);
	eyeballmatrix.translate(0.0, 0.0, 0.0);
}

Actor::~Actor(){
}

void Actor::setBody(GLuint b){
	body = b;
}

void Actor::setArms(GLuint a){
	arms = a;
}

void Actor::setLegs(GLuint l){
	legs = l;
}

void Actor::setHead(GLuint h){
	head = h;
}

void Actor::setHair(GLuint s){
	hair = s;
}

void Actor::setMouth(GLuint m){
	mouth = m;
}

void Actor::setEyes(GLuint e){
	eyes = e;
}

void Actor::setEyeballs(GLuint eb){
	eyeballs = eb;
}

void Actor::setBaseZScore(GLfloat zscore){
	this->zscore = zscore;
}

void Actor::mirror(){
	mirrorv = 1;
}

void Actor::handsUp(){
	handsup = 1;
}

void Actor::draw(ShaderProgram *prg, const char* texname, const char *matrixname){
	prg->use();

	prg->setParam("mirror", mirrorv);
	prg->setParam("zscore", zscore);
	prg->addTexture(texname, body);
	prg->setParam(matrixname, pmatrix);
	body_sp->draw();

	prg->setParam("zscore", zscore - 0.002f);
	if(handsup == 1){
		prg->setParam("up", 1);
	}
	prg->addTexture(texname, arms);
	prg->setParam(matrixname, pmatrix);
	arms_sp->draw();

	prg->setParam("up", 0);

	prg->setParam("zscore", zscore - 0.003f);
	prg->addTexture(texname, legs);
	prg->setParam(matrixname, pmatrix);
	legs_sp->draw();

	prg->setParam("zscore", zscore - 0.004f);
	prg->addTexture(texname, head);
	prg->setParam(matrixname, headmatrix * pmatrix);
	hair_sp->draw();

	prg->setParam("zscore", zscore - 0.005f);
	prg->addTexture(texname, hair);
	prg->setParam(matrixname, headmatrix * pmatrix);
	mouth_sp->draw();

	prg->setParam("zscore", zscore - 0.006f);
	prg->addTexture(texname, mouth);
	prg->setParam(matrixname, headmatrix * pmatrix);
	head_sp->draw();

	prg->setParam("zscore", zscore - 0.007f);
	prg->addTexture(texname, eyes);
	prg->setParam(matrixname, headmatrix * pmatrix);
	eyes_sp->draw();

	prg->setParam("zscore", zscore - 0.008f);
	prg->addTexture(texname, eyeballs);
	prg->setParam(matrixname, eyeballmatrix * headmatrix * pmatrix);
	eyeballs_sp->draw();
}

bool Actor::shouldTalk(double time){
	for(unsigned int i = 0; i < dialogstarts.size(); i++){
		if(time > dialogstarts.at(i) && time < dialogends.at(i)){
			return true;
		}
	}
	return false;
}

void Actor::movemouth(double time){
	if(time > epoch){
		epoch = time + drand48() / 2.0;
		int stage = rand() % mimiclen;
		setMouth(mimic[stage]);
		moveHead(drand48() * -0.05);
	}
}

void Actor::talk(double time, ShaderProgram *prg, const char* texname, const char *matrixname){
	if(shouldTalk(time)){
		movemouth(time);
	} else {
		setMouth(default_mouth);
	}

	draw(prg, texname, matrixname);
}

void Actor::loadDialog(string filename){
	ifstream dialog(filename, ios::in);
	string line;
	while(std::getline(dialog, line)){
		size_t pos = line.find("\t");
		string startstr = line.substr(0, pos);
		string endstr = line.substr(pos + 1);
		dialogstarts.push_back(atof(startstr.c_str()));
		dialogends.push_back(atof(endstr.c_str()));
	}
}

void Actor::setPos(GLfloat x, GLfloat y){
	pmatrix.modify(12, x);
	pmatrix.modify(13, y);
}

void Actor::rotate(GLfloat r){
	Matrix m;
	m.rotate(0.0, 0.0, r);
	pmatrix = m * pmatrix;
}

void Actor::setScale(GLfloat s){
	pmatrix.scale(0.3322 * s, 0.92 * s, 1.0);
};

void Actor::setMimic(GLuint a, GLuint b){
	mimic[0] = a;
	mimic[1] = b;
	mimiclen = 2;
}

void Actor::setMimic(GLuint a, GLuint b, GLuint c){
	mimic[0] = a;
	mimic[1] = b;
	mimic[2] = c;
	mimiclen = 3;
}

void Actor::setDefMouth(GLuint m){
	default_mouth = m;
}

void Actor::moveHead(GLfloat p){
	headmatrix.translate(0.0, p, 0.0);
}

void Actor::moveEyeball(GLfloat x, GLfloat y){
	eyeballmatrix.translate(x, y, 0.0);
}

Mulder::Mulder(Config *cfg){
	this->setBody(cfg->getTexture("mulder_body"));
	this->setArms(cfg->getTexture("mulder_arms"));
	this->setLegs(cfg->getTexture("legs_black"));
	this->setHead(cfg->getTexture("mulder_head"));
	this->setHair(cfg->getTexture("mulder_hair"));
	this->setMouth(cfg->getTexture("mulder_mouth_1"));
	this->setEyes(cfg->getTexture("mulder_eyes"));
	this->setEyeballs(cfg->getTexture("mulder_pupil"));
	this->setMimic(cfg->getTexture("mulder_mouth_2"), cfg->getTexture("mulder_mouth_3"), cfg->getTexture("mulder_mouth_4"));
	this->setDefMouth(cfg->getTexture("mulder_mouth_1"));
	this->loadDialog(cfg->getFile("mulderdialog"));
}

Scully::Scully(Config *cfg){
	this->setBody(cfg->getTexture("scully_body"));
	this->setArms(cfg->getTexture("scully_arms"));
	this->setLegs(cfg->getTexture("scully_legs"));
	this->setHead(cfg->getTexture("scully_head"));
	this->setHair(cfg->getTexture("scully_hair"));
	this->setMouth(cfg->getTexture("scully_mouth_1"));
	this->setEyes(cfg->getTexture("scully_eyes"));
	this->setEyeballs(cfg->getTexture("scully_pupil"));
	this->setMimic(cfg->getTexture("scully_mouth_2"), cfg->getTexture("scully_mouth_3"), cfg->getTexture("scully_mouth_4"));
	this->setDefMouth(cfg->getTexture("scully_mouth_1"));
	this->loadDialog(cfg->getFile("scullydialog"));
}

Woman::Woman(Config *cfg){
	this->setBody(cfg->getTexture("body_8"));
	this->setArms(cfg->getTexture("arms_8"));
	this->setLegs(cfg->getTexture("legs_6"));
	this->setHead(cfg->getTexture("head_2"));
	this->setHair(cfg->getTexture("hair_2"));
	this->setMouth(cfg->getTexture("mouth"));
	this->setEyes(cfg->getTexture("eyes"));
	this->setEyeballs(cfg->getTexture("pupil"));
	this->setMimic(cfg->getTexture("mouth_2"), cfg->getTexture("mouth_4"), cfg->getTexture("mouth_5"));
	this->setDefMouth(cfg->getTexture("mouth"));
	this->loadDialog(cfg->getFile("wifedialog"));
}

Widow::Widow(Config *cfg){	
	this->setBody(cfg->getTexture("body_9"));
	this->setArms(cfg->getTexture("arms_9"));
	this->setLegs(cfg->getTexture("legs_7"));
	this->setHead(cfg->getTexture("head_2"));
	this->setHair(cfg->getTexture("hair_7"));
	this->setMouth(cfg->getTexture("mouth_6"));
	this->setEyes(cfg->getTexture("eyes"));
	this->setEyeballs(cfg->getTexture("pupil"));
	this->setMimic(cfg->getTexture("mouth_2"), cfg->getTexture("mouth_4"), cfg->getTexture("mouth_5"));
	this->setDefMouth(cfg->getTexture("mouth_6"));
	this->loadDialog(cfg->getFile("wifedialog"));

}

Veryrich::Veryrich(Config *cfg){	
	this->setBody(cfg->getTexture("veryrich_body"));
	this->setArms(cfg->getTexture("veryrich_arms"));
	this->setLegs(cfg->getTexture("veryrich_legs"));
	this->setHead(cfg->getTexture("head_2"));
	this->setHair(cfg->getTexture("veryrich_hair"));
	this->setMouth(cfg->getTexture("mouth_5"));
	this->setEyes(cfg->getTexture("veryrich_eyes"));
	this->setEyeballs(cfg->getTexture("pupil"));
	this->loadDialog(cfg->getFile("veryrichdialog"));
	this->setMimic(cfg->getTexture("mouth_2"), cfg->getTexture("mouth_4"), cfg->getTexture("mouth_5"));
	this->setDefMouth(cfg->getTexture("mouth_5"));
	this->loadDialog(cfg->getFile("veryrichdialog"));

}

Shopassistant::Shopassistant(Config *cfg){	
	this->setBody(cfg->getTexture("shopassistant_body"));
	this->setArms(cfg->getTexture("shopassistant_arms"));
	this->setLegs(cfg->getTexture("legs_black"));
	this->setHead(cfg->getTexture("head_2"));
	this->setHair(cfg->getTexture("shopassistant_hair"));
	this->setMouth(cfg->getTexture("shopassistant_mouth_2"));
	this->setEyes(cfg->getTexture("shopassitant_eyes"));
	this->setEyeballs(cfg->getTexture("pupil"));
	this->setMimic(cfg->getTexture("shopassistant_mouth_2"), cfg->getTexture("shopassistant_mouth_1"));
	this->setDefMouth(cfg->getTexture("shopassistant_mouth_2"));
	this->loadDialog(cfg->getFile("sadialog"));

}

Programmer::Programmer(Config *cfg){
	this->setBody(cfg->getTexture("body_3"));
	this->setArms(cfg->getTexture("arms_3"));
	this->setLegs(cfg->getTexture("legs_black"));
	this->setHead(cfg->getTexture("head_2"));
	this->setHair(cfg->getTexture("hair_1"));
	this->setMouth(cfg->getTexture("mouth_6"));
	this->setEyes(cfg->getTexture("eyes"));
	this->setEyeballs(cfg->getTexture("pupil"));
	this->setMimic(cfg->getTexture("mouth_2"), cfg->getTexture("mouth_4"), cfg->getTexture("mouth_5"));
	this->setDefMouth(cfg->getTexture("mouth_6"));
	this->loadDialog(cfg->getFile("progdialog"));
}

Zombie::Zombie(Config *cfg){	
	this->setBody(cfg->getTexture("body_4"));
	this->setArms(cfg->getTexture("arms_4"));
	this->setLegs(cfg->getTexture("legs_black"));
	this->setHead(cfg->getTexture("zombie_head"));
	this->setHair(cfg->getTexture("shopassistant_hair"));
	this->setMouth(cfg->getTexture("mouth_8"));
	this->setEyes(cfg->getTexture("eyes_5"));
	this->setEyeballs(cfg->getTexture("pupil"));
}
