#pragma once

#include "CGengine.hpp"
#include <vector>

using namespace cg_engine;

class Actor {
	public:
		Actor();
		~Actor();
		void draw(ShaderProgram *prg, const char* texname, const char* matrixname);
		void talk(double time, ShaderProgram *prg, const char* texname, const char *matrixname);
		void setBody(GLuint b);
		void setArms(GLuint a);
		void setLegs(GLuint l);
		void setHair(GLuint s);
		void setHead(GLuint h);
		void setMouth(GLuint m);
		void setEyes(GLuint e);
		void setEyeballs(GLuint eb);
		void loadDialog(string filename);
		void setMimic(GLuint a, GLuint b);
		void setMimic(GLuint a, GLuint b, GLuint c);
		void setDefMouth(GLuint m);
		void setPos(GLfloat x, GLfloat y);
		void rotate(GLfloat r);
		void setScale(GLfloat s);
		void setBaseZScore(GLfloat zscore);
		void mirror();
		void handsUp();
		void moveHead(GLfloat p);
		void moveEyeball(GLfloat x, GLfloat y);
	private:
		bool shouldTalk(double time);
		void movemouth(double time);
		Sprite *body_sp;
		Sprite *arms_sp;
		Sprite *legs_sp;
		Sprite *head_sp;
		Sprite *hair_sp;
		Sprite *mouth_sp;
		Sprite *eyes_sp;
		Sprite *eyeballs_sp;

		GLuint body;
		GLuint arms;
		GLuint legs;
		GLuint head;
		GLuint hair;
		GLuint mouth;
		GLuint eyes;
		GLuint eyeballs;
		GLuint mimic[3];
		GLuint default_mouth;
		int mirrorv = 0;
		int handsup = 0;

		unsigned int mimiclen = 1;

		Matrix pmatrix;
		Matrix headmatrix;
		Matrix eyeballmatrix;
		Matrix legmatrix;

		double epoch = 0.0;
		GLfloat zscore = 0.11f;
		std::vector<double> dialogstarts;
		std::vector<double> dialogends;
};

class Mulder : public Actor {
	public:
		Mulder(Config *cfg);
};

class Scully : public Actor {
	public:
		Scully(Config *cfg);
};

class Veryrich : public Actor {
	public:
		Veryrich(Config *cfg);
};

class Programmer : public Actor {
	public:
		Programmer(Config *cfg);
};

class Wife : public Actor {
	public:
		Wife(Config *cfg);
};

class Zombie : public Actor {
	public:
		Zombie(Config *cfg);
};

class Shopassistant : public Actor {
	public:
		Shopassistant(Config *cfg);
};

class Widow : public Actor {
	public:
		Widow(Config *cfg);
};

class Woman : public Actor {
	public:
		Woman(Config *cfg);
};
