#include "taz.hpp"
#include <SDL2/SDL.h>

using namespace cg_engine;

#define MOVERANGE -0.1
#define MOVEEYERANGE 0.04

IntroScene::IntroScene(Config *conf, int, int){
	start = 0.0;
	end   = 5.417;

	def    = conf->getShader("default");
	layout = new Sprite();
	layouttex = conf->getTexture("intro");
}

void IntroScene::show(double){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);
	layout->draw();
}

HomeScene::HomeScene(Config *conf, int, int){
	start = 5.417;
	end   = 30.998719;

	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("home");
	wife = new Woman(conf);
	programmer = new Programmer(conf);
	programmer->loadDialog("data/programmer.dialog");
}

void HomeScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);
	layout->draw();

	programmer->setScale(0.5);
	programmer->moveEyeball(-0.002083, -0.048148);
	programmer->setPos(-0.170833, -0.259259);
	programmer->talk(time, sprite, "img", "pmatrix");

	wife->setBaseZScore(0.10);
	wife->moveEyeball(-0.041667, 0.001852);
	wife->setScale(0.5);
	wife->setPos(0.054167, -0.262963);
	wife->talk(time, sprite, "img", "pmatrix");
}

StuffScene::StuffScene(Config *conf, int, int){
	start = 30.998719;
	end   = 46.396776;

	layout = new Sprite();
	def = conf->getShader("default");
	layouttex = conf->getTexture("packs");
}

void StuffScene::show(double){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();
}

TelephoneScene::TelephoneScene(Config *conf, int, int){
	start = 46.396776;
	end   = 58.472094;

	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("home");

	wife = new Woman(conf);
	wife->setArms(conf->getTexture("arms_telo"));
}

void TelephoneScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	wife->setScale(0.5);
	wife->setPos(0.054167, -0.262963);
	wife->talk(time, sprite, "img", "pmatrix");
}

FuneralScene::FuneralScene(Config *conf, int, int){
	start = 58.472094;
	end   = 99.763199;
	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("cemetry_day");

	mulder = new Mulder(conf);
	scully = new Scully(conf);
	widow  = new Widow(conf);
}

void FuneralScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();
	mulder->setScale(0.4);
	mulder->setPos(0.470833, -0.377778);
	mulder->talk(time, sprite, "img", "pmatrix");

	widow->setScale(0.4);
	widow->setPos(0.084375, -0.392593);
	widow->moveEyeball(0.047917, 0.005556);
	widow->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.4);
	scully->moveEyeball(-0.041667, 0.001852);
	scully->setPos(0.747917, -0.375926);
	scully->talk(time, sprite, "img", "pmatrix");
}

CemetryNightScene::CemetryNightScene(Config *conf, int, int){
	start = 99.763199;
	end   = 105.962943;
	layout = new Sprite();
	def = conf->getShader("default");
	layouttex = conf->getTexture("cemetry_night");
	surprisearm = conf->getTexture("surprisearm");
}

void CemetryNightScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	if(time > 104.0){
		def->addTexture("img", surprisearm);
	} else {
		def->addTexture("img", layouttex);
	}

	layout->draw();
}

CemetryDayScene::CemetryDayScene(Config *conf, int, int){
	start = 105.962943;
	end   = 125.251035;

	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("cemetry_day2");

	mulder = new Mulder(conf);
	scully = new Scully(conf);
}

void CemetryDayScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	mulder->setScale(0.4);
	mulder->moveEyeball(-0.011458 + sin(time) / 25.0, -0.018519);
	mulder->setPos(0.470833, -0.377778);
	mulder->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.4);
	scully->moveEyeball(-0.023958 + cos(time) / 30.0, -0.027778);
	scully->setPos(0.747917, -0.375926);
	scully->talk(time, sprite, "img", "pmatrix");
}

VeryrichOfficeScene::VeryrichOfficeScene(Config *conf, int, int){
	start = 125.251035;
	end   = 168.851848;
	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("office");
	mulder = new Mulder(conf);
	scully = new Scully(conf);
	veryrich = new Veryrich(conf);
}

void VeryrichOfficeScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	mulder->setScale(0.7);
	mulder->moveEyeball(0.034375, 0.001852);
	mulder->setPos(-0.275000, -0.192593);
	mulder->mirror();
	mulder->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.7);
	scully->moveEyeball(0.045833, -0.003704);
	scully->setPos(-0.753125, -0.21111);
	scully->talk(time, sprite, "img", "pmatrix");

	veryrich->setScale(0.7);
	veryrich->moveEyeball(-0.031250, -0.011111);
	veryrich->setPos(0.592708, -0.183333);
	veryrich->talk(time, sprite, "img", "pmatrix");
}

OfficeScene::OfficeScene(Config *conf, int, int){
	start = 168.851848;
	end   = 180.157;

	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("office2");

	mulder = new Mulder(conf);
	scully = new Scully(conf);
	wife   = new Woman(conf);
	wife->setDefMouth(conf->getTexture("mouth_6"));
}

void OfficeScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	mulder->setScale(0.8);
	mulder->moveEyeball(-0.022917, 0.007407);
	mulder->setPos(0.118750, -0.075926);
	mulder->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.8);
	scully->moveEyeball(-0.044792, -0.001852);
	scully->setPos(0.664583, -0.075926);
	scully->talk(time, sprite, "img", "pmatrix");

	wife->setScale(0.8);
	wife->moveEyeball(0.034375, -0.007407);
	wife->setPos(-0.687500, -0.075926);
	wife->talk(time, sprite, "img", "pmatrix");
}

AmigaOnDeskScene::AmigaOnDeskScene(Config *conf, int, int){
	start = 180.157;
	end   = 192.273103;
	layout = new Sprite();
	def = conf->getShader("default");
	layouttex = conf->getTexture("desk");
}

void AmigaOnDeskScene::show(double){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);
	layout->draw();
}

ShopScene::ShopScene(Config *conf, int, int){
	start = 192.273103;
	end   = 207.711681;

	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	layouttex = conf->getTexture("computershop");

	mulder = new Mulder(conf);
	scully = new Scully(conf);
	shopassistant = new Shopassistant(conf);
}

void ShopScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	mulder->setScale(0.5);
	mulder->moveEyeball(-0.026042, 0.000000);
	mulder->setPos(0.595833, -0.312963);

	if(time > 204){
		mulder->mirror();
		mulder->moveEyeball(0.028125, -0.016071);
	}
	mulder->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.5);
	scully->moveEyeball(-0.050000, 0.000000);
	scully->setPos(0.210417, -0.348148);
	scully->talk(time, sprite, "img", "pmatrix");

	shopassistant->setScale(0.5);
	shopassistant->moveEyeball(0.019792, -0.020370);
	shopassistant->setPos(-0.362500, -0.305556);
	shopassistant->talk(time, sprite, "img", "pmatrix");
}

PartyScene::PartyScene(Config *conf, int, int){
	start = 207.711681;
	end   = 280.244632;
	layout = new Sprite();
	def = conf->getShader("default");
	sprite = conf->getShader("sprite");
	bloodshader = conf->getShader("blood");

	layouttex = conf->getTexture("party");
	deadeye = conf->getTexture("dead_eyes");
	deadmouth = conf->getTexture("dead_mouth");

	mulder = new Mulder(conf);
	scully = new Scully(conf);
	veryrich = new Veryrich(conf);
	zombie1 = new Zombie(conf);
	zombie2 = new Zombie(conf);
	zombie3 = new Zombie(conf);

	bloodparticle = new Particle(60, 0.605417, -0.220259, 0.0);
}

void PartyScene::show(double time){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	def->use();
	def->addTexture("img", layouttex);

	layout->draw();

	mulder->setScale(0.4);
	mulder->mirror();
	mulder->setPos(-0.542708, -0.544444);
	mulder->talk(time, sprite, "img", "pmatrix");

	scully->setScale(0.4);
	scully->setPos(-0.275000, -0.542593);
	scully->talk(time, sprite, "img", "pmatrix");

	veryrich->setScale(0.4);
	if(time < 218.926){
		veryrich->setPos(0.855208 + 218.926 - time, -0.538889 + sin(time*100.0) / 50.0);
	} else {
		veryrich->setPos(0.855208, -0.538889);
	}
	if(time > 269.243){
		veryrich->setEyes(deadeye);
		veryrich->setMouth(deadmouth);
	}

	if(time > 263.0){
		veryrich->setPos(0.600000, -0.214815);
		veryrich->rotate(4.76f);
	}

	if(time > 269.243 && time < 271.472){
		veryrich->moveHead( (time - 269.243) * 2.0 );
		veryrich->draw(sprite, "img", "pmatrix");

		bloodshader->use();
		Matrix p;
		p.translate(0.0, 0.0, 0.0);
		bloodshader->setParam("pmatrix", p);
		bloodshader->setParam("time", (float)time);
		bloodparticle->update(time);
		bloodparticle->draw();
	} else {
		veryrich->talk(time, sprite, "img", "pmatrix");
	}

	zombie1->setScale(0.4);
	zombie2->setScale(0.4);
	zombie3->setScale(0.4);
	zombie2->setBaseZScore(0.1);
	zombie3->setBaseZScore(0.09);
	if(time > 263.0){
		//zombie1->handsUp();
		zombie2->handsUp();
		zombie3->handsUp();
	}
	zombie1->setPos(0.167708, -0.542593);
	zombie2->setPos(0.435417, -0.542593);
	zombie3->setPos(0.700000, -0.544444);
	if(time > zdelta1){
		zdelta1 = time + drand48() * 1.2;
		zombie1->moveHead(drand48() * MOVERANGE);
		zombie1->moveEyeball(drand48() * MOVEEYERANGE - 0.02, drand48() * MOVEEYERANGE - 0.02);
	}
	if(time > zdelta2){
		zdelta2 = time + drand48() * 1.2;
		zombie2->moveHead(drand48() * MOVERANGE);
		zombie2->moveEyeball(drand48() * MOVEEYERANGE - 0.02, drand48() * MOVEEYERANGE - 0.02);
	}
	if(time > zdelta3){
		zdelta3 = time + drand48() * 1.2;
		zombie3->moveHead(drand48() * MOVERANGE);
		zombie3->moveEyeball(drand48() * MOVEEYERANGE - 0.02, drand48() * MOVEEYERANGE - 0.02);
	}
	zombie1->draw(sprite, "img", "pmatrix");
	zombie2->draw(sprite, "img", "pmatrix");
	zombie3->draw(sprite, "img", "pmatrix");
}

EndcreditScene::EndcreditScene(Config *conf, int, int){
	start = 280.244632;
	end   = 286.403855;
	layout = new Sprite();
	layouttex = conf->getTexture("endcredit");
	def = conf->getShader("default");
}

void EndcreditScene::show(double){
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	def->use();
	def->addTexture("img", layouttex);
	layout->draw();
}

MyDemo::MyDemo(int w, int h): Demo(w, h){
	glClearColor(1.0, 1.0, 1.0, 1.0);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_PROGRAM_POINT_SIZE);
}

void MyDemo::init(){
	Config *conf = new Config("data/config.txt");

	scenes.push_back(new IntroScene(conf, screenwidth, screenheight));
	scenes.push_back(new HomeScene(conf, screenwidth, screenheight));
	scenes.push_back(new StuffScene(conf, screenwidth, screenheight));
	scenes.push_back(new TelephoneScene(conf, screenwidth, screenheight));
	scenes.push_back(new FuneralScene(conf, screenwidth, screenheight));
	scenes.push_back(new CemetryNightScene(conf, screenwidth, screenheight));
	scenes.push_back(new CemetryDayScene(conf, screenwidth, screenheight));
	scenes.push_back(new VeryrichOfficeScene(conf, screenwidth, screenheight));
	scenes.push_back(new OfficeScene(conf, screenwidth, screenheight));
	scenes.push_back(new AmigaOnDeskScene(conf, screenwidth, screenheight));
	scenes.push_back(new ShopScene(conf, screenwidth, screenheight));
	scenes.push_back(new PartyScene(conf, screenwidth, screenheight));
	scenes.push_back(new EndcreditScene(conf, screenwidth, screenheight));
}

